@@ -1,9 +1,7 @@
 //! Compare the speed of rust hash implementations
 
-use ahash;
-use fxhash;
 use std::hash::Hasher;
-use xxhash_rust::const_xxh3;
+//use xxhash_rust::const_xxh3;
 use xxhash_rust::xxh3;
 
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
@@ -19,14 +17,14 @@ fn criterion_benchmark(c: &mut Criterion) {
     c.bench_function("xxh3", |b| {
         b.iter(|| xxh3::xxh3_64_with_seed(black_box(&bench_vec), 0))
     });
-    c.bench_function("const_xxh3", |b| {
+    /*c.bench_function("const_xxh3", |b| {
         b.iter(|| const_xxh3::xxh3_64_with_seed(black_box(&bench_vec), 0))
-    });
+    });*/
     c.bench_function("ahash", |b| {
         b.iter(|| {
             let mut hasher = ahash::AHasher::new_with_keys(123, 456);
             hasher.write(black_box(&bench_vec));
-            hasher.finish();
+            hasher.finish()
         })
     });
     c.bench_function("fxhash", |b| {
