@@ -1,62 +1,68 @@
-#[cfg(feature = "std")]
-use serde::de::DeserializeOwned;
+//! The [`Launcher`] launches multiple fuzzer instances in parallel.
+//! Thanks to it, we won't need a `for` loop in a shell script...
+//!
+//! To use multiple [`Launcher`]`s` for individual configurations,
+//! we can set `spawn_broker` to `false` on all but one.
+//!
+//! To connect multiple nodes together via TCP, we can use the `remote_broker_addr`.
+//! (this requires the `llmp_bind_public` compile-time feature for `LibAFL`).
+//!
+//! On `Unix` systems, the [`Launcher`] will use `fork` if the `fork` feature is used for `LibAFL`.
+//! Else, it will start subsequent nodes with the same commandline, and will set special `env` variables accordingly.
 
+#[cfg(all(feature = "std", any(windows, not(feature = "fork"))))]
+use crate::bolts::os::startable_self;
+#[cfg(all(unix, feature = "std", feature = "fork"))]
+use crate::bolts::os::{dup2, fork, ForkResult};
 #[cfg(feature = "std")]
 use crate::{
     bolts::shmem::ShMemProvider,
-    events::{LlmpRestartingEventManager, ManagerKind, RestartingMgr},
+    events::{EventConfig, LlmpRestartingEventManager, ManagerKind, RestartingMgr},
     inputs::Input,
+    monitors::Monitor,
     observers::ObserversTuple,
-    stats::Stats,
     Error,
 };
 
-#[cfg(all(windows, feature = "std"))]
-use crate::bolts::os::startable_self;
-
-#[cfg(all(unix, feature = "std"))]
-use crate::bolts::os::{dup2, fork, ForkResult};
-
-#[cfg(all(unix, feature = "std"))]
-use std::{fs::File, os::unix::io::AsRawFd};
-
+#[cfg(feature = "std")]
+use core::marker::PhantomData;
+#[cfg(all(feature = "std", any(windows, not(feature = "fork"))))]
+use core_affinity::CoreId;
+#[cfg(feature = "std")]
+use serde::de::DeserializeOwned;
 #[cfg(feature = "std")]
 use std::net::SocketAddr;
-#[cfg(all(windows, feature = "std"))]
+#[cfg(all(feature = "std", any(windows, not(feature = "fork"))))]
 use std::process::Stdio;
-
-#[cfg(all(windows, feature = "std"))]
-use core_affinity::CoreId;
-
+#[cfg(all(unix, feature = "std", feature = "fork"))]
+use std::{fs::File, os::unix::io::AsRawFd};
 #[cfg(feature = "std")]
 use typed_builder::TypedBuilder;
 
-/// The Launcher client callback type reference
-#[cfg(feature = "std")]
-pub type LauncherClientFnRef<'a, I, OT, S, SP> =
-    &'a mut dyn FnMut(Option<S>, LlmpRestartingEventManager<I, OT, S, SP>) -> Result<(), Error>;
-
+/// The (internal) `env` that indicates we're running as client.
 const _AFL_LAUNCHER_CLIENT: &str = "AFL_LAUNCHER_CLIENT";
 /// Provides a Launcher, which can be used to launch a fuzzing run on a specified list of cores
 #[cfg(feature = "std")]
 #[derive(TypedBuilder)]
 #[allow(clippy::type_complexity)]
-pub struct Launcher<'a, I, OT, S, SP, ST>
+pub struct Launcher<'a, CF, I, MT, OT, S, SP>
 where
-    I: Input,
-    ST: Stats,
+    CF: FnOnce(Option<S>, LlmpRestartingEventManager<I, OT, S, SP>, usize) -> Result<(), Error>,
+    I: Input + 'a,
+    MT: Monitor,
     SP: ShMemProvider + 'static,
-    OT: ObserversTuple<I, S>,
-    S: DeserializeOwned,
+    OT: ObserversTuple<I, S> + 'a,
+    S: DeserializeOwned + 'a,
 {
     /// The ShmemProvider to use
     shmem_provider: SP,
-    /// The stats instance to use
-    stats: ST,
+    /// The monitor instance to use
+    monitor: MT,
     /// The configuration
-    configuration: String,
+    configuration: EventConfig,
     /// The 'main' function to run for each client forked. This probably shouldn't return
-    run_client: LauncherClientFnRef<'a, I, OT, S, SP>,
+    #[builder(default, setter(strip_option))]
+    run_client: Option<CF>,
     /// The broker port to use (or to attach to, in case [`Self::spawn_broker`] is `false`)
     #[builder(default = 1337_u16)]
     broker_port: u16,
@@ -75,33 +81,41 @@ where
     /// Then, clients launched by this [`Launcher`] can connect to the original `broker`.
     #[builder(default = true)]
     spawn_broker: bool,
+    #[builder(setter(skip), default = PhantomData)]
+    phantom_data: PhantomData<(&'a I, &'a OT, &'a S, &'a SP)>,
 }
 
 #[cfg(feature = "std")]
-impl<'a, I, OT, S, SP, ST> Launcher<'a, I, OT, S, SP, ST>
+impl<'a, CF, I, MT, OT, S, SP> Launcher<'a, CF, I, MT, OT, S, SP>
 where
+    CF: FnOnce(Option<S>, LlmpRestartingEventManager<I, OT, S, SP>, usize) -> Result<(), Error>,
     I: Input,
     OT: ObserversTuple<I, S> + serde::de::DeserializeOwned,
-    ST: Stats + Clone,
+    MT: Monitor + Clone,
     SP: ShMemProvider + 'static,
     S: DeserializeOwned,
 {
     /// Launch the broker and the clients and fuzz
-    #[cfg(all(unix, feature = "std"))]
+    #[cfg(all(unix, feature = "std", feature = "fork"))]
     #[allow(clippy::similar_names)]
     pub fn launch(&mut self) -> Result<(), Error> {
+        if self.run_client.is_none() {
+            return Err(Error::IllegalArgument(
+                "No client callback provided".to_string(),
+            ));
+        }
+
         let core_ids = core_affinity::get_core_ids().unwrap();
         let num_cores = core_ids.len();
         let mut handles = vec![];
 
         println!("spawning on cores: {:?}", self.cores);
-        let file = self
-            .stdout_file
-            .map(|filename| File::create(filename).unwrap());
 
         // Spawn clients
+        let mut index = 0_u64;
         for (id, bind_to) in core_ids.iter().enumerate().take(num_cores) {
             if self.cores.iter().any(|&x| x == id) {
+                index += 1;
                 self.shmem_provider.pre_fork()?;
                 match unsafe { fork() }? {
                     ForkResult::Parent(child) => {
@@ -115,25 +129,27 @@ where
                         self.shmem_provider.post_fork(true)?;
 
                         #[cfg(feature = "std")]
-                        std::thread::sleep(std::time::Duration::from_secs((id + 1) as u64));
+                        std::thread::sleep(std::time::Duration::from_millis(index * 100));
 
                         #[cfg(feature = "std")]
-                        if file.is_some() {
-                            dup2(file.as_ref().unwrap().as_raw_fd(), libc::STDOUT_FILENO)?;
-                            dup2(file.as_ref().unwrap().as_raw_fd(), libc::STDERR_FILENO)?;
+                        if let Some(filename) = self.stdout_file {
+                            let file = File::create(filename).unwrap();
+                            dup2(file.as_raw_fd(), libc::STDOUT_FILENO)?;
+                            dup2(file.as_raw_fd(), libc::STDERR_FILENO)?;
                         }
                         // Fuzzer client. keeps retrying the connection to broker till the broker starts
-                        let (state, mgr) = RestartingMgr::<I, OT, S, SP, ST>::builder()
+                        let (state, mgr) = RestartingMgr::<I, MT, OT, S, SP>::builder()
                             .shmem_provider(self.shmem_provider.clone())
                             .broker_port(self.broker_port)
                             .kind(ManagerKind::Client {
                                 cpu_core: Some(*bind_to),
                             })
-                            .configuration(self.configuration.clone())
+                            .configuration(self.configuration)
                             .build()
                             .launch()?;
 
-                        (self.run_client)(state, mgr)?;
+                        (self.run_client.take().unwrap())(state, mgr, bind_to.id)
+                            .expect("Client closure failed");
                         break;
                     }
                 };
@@ -145,13 +161,13 @@ where
             println!("I am broker!!.");
 
             // TODO we don't want always a broker here, think about using different laucher process to spawn different configurations
-            RestartingMgr::<I, OT, S, SP, ST>::builder()
+            RestartingMgr::<I, MT, OT, S, SP>::builder()
                 .shmem_provider(self.shmem_provider.clone())
-                .stats(Some(self.stats.clone()))
+                .monitor(Some(self.monitor.clone()))
                 .broker_port(self.broker_port)
                 .kind(ManagerKind::Broker)
                 .remote_broker_addr(self.remote_broker_addr)
-                .configuration(self.configuration.clone())
+                .configuration(self.configuration)
                 .build()
                 .launch()?;
 
@@ -178,29 +194,30 @@ where
     }
 
     /// Launch the broker and the clients and fuzz
-    #[cfg(all(windows, feature = "std"))]
-    #[allow(unused_mut)]
+    #[cfg(all(feature = "std", any(windows, not(feature = "fork"))))]
+    #[allow(unused_mut, clippy::match_wild_err_arm)]
     pub fn launch(&mut self) -> Result<(), Error> {
         let is_client = std::env::var(_AFL_LAUNCHER_CLIENT);
 
         let mut handles = match is_client {
             Ok(core_conf) => {
+                let core_id = core_conf.parse()?;
+
                 //todo: silence stdout and stderr for clients
 
                 // the actual client. do the fuzzing
-                let (state, mgr) = RestartingMgr::<I, OT, S, SP, ST>::builder()
+                let (state, mgr) = RestartingMgr::<I, MT, OT, S, SP>::builder()
                     .shmem_provider(self.shmem_provider.clone())
                     .broker_port(self.broker_port)
                     .kind(ManagerKind::Client {
-                        cpu_core: Some(CoreId {
-                            id: core_conf.parse()?,
-                        }),
+                        cpu_core: Some(CoreId { id: core_id }),
                     })
-                    .configuration(self.configuration.clone())
+                    .configuration(self.configuration)
                     .build()
                     .launch()?;
 
-                (self.run_client)(state, mgr)?;
+                (self.run_client.take().unwrap())(state, mgr, core_id)
+                    .expect("Client closure failed");
 
                 unreachable!("Fuzzer client code should never get here!");
             }
@@ -221,19 +238,15 @@ where
                 //spawn clients
                 for (id, _) in core_ids.iter().enumerate().take(num_cores) {
                     if self.cores.iter().any(|&x| x == id) {
-                        for id in 0..num_cores {
-                            let stdio = if self.stdout_file.is_some() {
-                                Stdio::inherit()
-                            } else {
-                                Stdio::null()
-                            };
+                        let stdio = if self.stdout_file.is_some() {
+                            Stdio::inherit()
+                        } else {
+                            Stdio::null()
+                        };
 
-                            if self.cores.iter().any(|&x| x == id) {
-                                std::env::set_var(_AFL_LAUNCHER_CLIENT, id.to_string());
-                                let child = startable_self()?.stdout(stdio).spawn()?;
-                                handles.push(child);
-                            }
-                        }
+                        std::env::set_var(_AFL_LAUNCHER_CLIENT, id.to_string());
+                        let child = startable_self()?.stdout(stdio).spawn()?;
+                        handles.push(child);
                     }
                 }
 
@@ -246,13 +259,13 @@ where
             #[cfg(feature = "std")]
             println!("I am broker!!.");
 
-            RestartingMgr::<I, OT, S, SP, ST>::builder()
+            RestartingMgr::<I, MT, OT, S, SP>::builder()
                 .shmem_provider(self.shmem_provider.clone())
-                .stats(Some(self.stats.clone()))
+                .monitor(Some(self.monitor.clone()))
                 .broker_port(self.broker_port)
                 .kind(ManagerKind::Broker)
                 .remote_broker_addr(self.remote_broker_addr)
-                .configuration(self.configuration.clone())
+                .configuration(self.configuration)
                 .build()
                 .launch()?;
 
