@@ -26,53 +26,45 @@ pub type StdShMemProvider = Win32ShMemProvider;
 pub type StdShMem = Win32ShMem;
 
 #[cfg(all(target_os = "android", feature = "std"))]
-pub type StdShMemProvider = RcShMemProvider<ServedShMemProvider<AshmemShMemProvider>>;
+pub type StdShMemProvider =
+    RcShMemProvider<ServedShMemProvider<unix_shmem::ashmem::AshmemShMemProvider>>;
 #[cfg(all(target_os = "android", feature = "std"))]
-pub type StdShMem = RcShMem<ServedShMemProvider<AshmemShMemProvider>>;
+pub type StdShMem = RcShMem<ServedShMemProvider<unix_shmem::ashmem::AshmemShMemProvider>>;
 #[cfg(all(target_os = "android", feature = "std"))]
-pub type StdShMemService = ShMemService<AshmemShMemProvider>;
+pub type StdShMemService = ShMemService<unix_shmem::ashmem::AshmemShMemProvider>;
 
-#[cfg(all(feature = "std", any(target_os = "ios", target_os = "macos")))]
+#[cfg(all(feature = "std", target_vendor = "apple"))]
 pub type StdShMemProvider = RcShMemProvider<ServedShMemProvider<MmapShMemProvider>>;
-#[cfg(all(feature = "std", any(target_os = "ios", target_os = "macos")))]
+#[cfg(all(feature = "std", target_vendor = "apple"))]
 pub type StdShMem = RcShMem<ServedShMemProvider<MmapShMemProvider>>;
-#[cfg(all(feature = "std", any(target_os = "ios", target_os = "macos")))]
+#[cfg(all(feature = "std", target_vendor = "apple"))]
 pub type StdShMemService = ShMemService<MmapShMemProvider>;
 
 /// The default [`ShMemProvider`] for this os.
 #[cfg(all(
     feature = "std",
     unix,
-    not(any(target_os = "android", target_os = "ios", target_os = "macos"))
+    not(any(target_os = "android", target_vendor = "apple"))
 ))]
 pub type StdShMemProvider = UnixShMemProvider;
 /// The default [`ShMemProvider`] for this os.
 #[cfg(all(
     feature = "std",
     unix,
-    not(any(target_os = "android", target_os = "ios", target_os = "macos"))
+    not(any(target_os = "android", target_vendor = "apple"))
 ))]
 pub type StdShMem = UnixShMem;
 
 #[cfg(any(
-    not(any(target_os = "android", target_os = "macos", target_os = "ios")),
+    not(any(target_os = "android", target_vendor = "apple")),
     not(feature = "std")
 ))]
 pub type StdShMemService = DummyShMemService;
 
 use serde::{Deserialize, Serialize};
 #[cfg(feature = "std")]
-use std::{
-    convert::{TryFrom, TryInto},
-    env,
-};
+use std::env;
 
-#[cfg(all(
-    unix,
-    feature = "std",
-    any(target_os = "ios", target_os = "macos", target_os = "android")
-))]
-use super::os::unix_shmem_server::ServedShMem;
 #[cfg(all(unix, feature = "std"))]
 use crate::bolts::os::pipes::Pipe;
 #[cfg(all(unix, feature = "std"))]
@@ -203,7 +195,7 @@ pub trait ShMem: Sized + Debug + Clone {
 /// A [`ShMemProvider`] provides access to shared maps.
 /// They are the backbone of [`crate::bolts::llmp`] for inter-process communication.
 /// All you need for scaling on a new target is to implement this interface, as well as the respective [`ShMem`].
-pub trait ShMemProvider: Send + Clone + Default + Debug {
+pub trait ShMemProvider: Clone + Default + Debug {
     /// The actual shared map handed out by this [`ShMemProvider`].
     type Mem: ShMem;
 
@@ -316,8 +308,8 @@ where
     parent_child_pipe: Option<Pipe>,
 }
 
-#[cfg(all(unix, feature = "std"))]
-unsafe impl<SP: ShMemProvider> Send for RcShMemProvider<SP> {}
+//#[cfg(all(unix, feature = "std"))]
+//unsafe impl<SP: ShMemProvider> Send for RcShMemProvider<SP> {}
 
 #[cfg(all(unix, feature = "std"))]
 impl<SP> ShMemProvider for RcShMemProvider<SP>
@@ -372,8 +364,8 @@ where
     fn post_fork(&mut self, is_child: bool) -> Result<(), Error> {
         if is_child {
             self.await_parent_done()?;
-            let child_shmem = self.internal.borrow_mut().clone();
-            self.internal = Rc::new(RefCell::new(child_shmem));
+            //let child_shmem = self.internal.borrow_mut().clone();
+            //self.internal = Rc::new(RefCell::new(child_shmem));
         }
         self.internal.borrow_mut().post_fork(is_child)?;
         if is_child {
@@ -471,7 +463,7 @@ where
 pub mod unix_shmem {
     /// Shared memory provider for Android, allocating and forwarding maps over unix domain sockets.
     #[cfg(target_os = "android")]
-    pub type UnixShMemProvider = ashmem::ServedShMemProvider;
+    pub type UnixShMemProvider = ashmem::AshmemShMemProvider;
     /// Shared memory for Android
     #[cfg(target_os = "android")]
     pub type UnixShMem = ashmem::AshmemShMem;
@@ -492,7 +484,7 @@ pub mod unix_shmem {
     #[cfg(all(unix, feature = "std", not(target_os = "android")))]
     mod default {
 
-        use core::{convert::TryInto, ptr, slice};
+        use core::{ptr, slice};
         use libc::{
             c_int, c_long, c_uchar, c_uint, c_ulong, c_ushort, close, ftruncate, mmap, munmap,
             perror, shm_open, shm_unlink, shmat, shmctl, shmget,
@@ -708,16 +700,18 @@ pub mod unix_shmem {
         impl Drop for MmapShMem {
             fn drop(&mut self) {
                 unsafe {
-                    if self.map.is_null() {
-                        panic!("Map should never be null for MmapShMem (on Drop)");
-                    }
+                    assert!(
+                        !self.map.is_null(),
+                        "Map should never be null for MmapShMem (on Drop)"
+                    );
 
                     munmap(self.map as *mut _, self.map_size);
                     self.map = ptr::null_mut();
 
-                    if self.shm_fd == -1 {
-                        panic!("FD should never be -1 for MmapShMem (on Drop)");
-                    }
+                    assert!(
+                        self.shm_fd != -1,
+                        "FD should never be -1 for MmapShMem (on Drop)"
+                    );
 
                     // None in case we didn't [`shm_open`] this ourselves, but someone sent us the FD.
                     if let Some(filename_path) = self.filename_path {
@@ -915,7 +909,7 @@ pub mod unix_shmem {
                     //return Err(Error::Unknown("Failed to set the ashmem mapping's name".to_string()));
                     //};
 
-                    if ioctl(fd, ASHMEM_SET_SIZE, map_size) != 0 {
+                    if ioctl(fd, ASHMEM_SET_SIZE as _, map_size) != 0 {
                         close(fd);
                         return Err(Error::Unknown(
                             "Failed to set the ashmem mapping's size".to_string(),
@@ -950,7 +944,7 @@ pub mod unix_shmem {
                 unsafe {
                     let fd: i32 = id.to_string().parse().unwrap();
                     #[allow(clippy::cast_sign_loss)]
-                    if ioctl(fd, ASHMEM_GET_SIZE) as u32 as usize != map_size {
+                    if ioctl(fd, ASHMEM_GET_SIZE as _) as u32 as usize != map_size {
                         return Err(Error::Unknown(
                             "The mapping's size differs from the requested size".to_string(),
                         ));
@@ -1007,14 +1001,14 @@ pub mod unix_shmem {
                     let fd: i32 = self.id.to_string().parse().unwrap();
 
                     #[allow(clippy::cast_sign_loss)]
-                    let length = ioctl(fd, ASHMEM_GET_SIZE) as u32;
+                    let length = ioctl(fd, ASHMEM_GET_SIZE as _) as u32;
 
                     let ap = ashmem_pin {
                         offset: 0,
                         len: length,
                     };
 
-                    ioctl(fd, ASHMEM_UNPIN, &ap);
+                    ioctl(fd, ASHMEM_UNPIN as _, &ap);
                     close(fd);
                 }
             }
