@@ -5,30 +5,34 @@ pub use simple::*;
 pub mod llmp;
 pub use llmp::*;
 
+use ahash::AHasher;
 use alloc::{string::String, vec::Vec};
-use core::{fmt, marker::PhantomData, time::Duration};
+use core::{fmt, hash::Hasher, marker::PhantomData, time::Duration};
 use serde::{Deserialize, Serialize};
 
+#[cfg(feature = "std")]
+use uuid::Uuid;
+
 use crate::{
-    executors::ExitKind, inputs::Input, observers::ObserversTuple, stats::UserStats, Error,
+    bolts::current_time,
+    executors::ExitKind,
+    inputs::Input,
+    monitors::UserStats,
+    observers::ObserversTuple,
+    state::{HasClientPerfMonitor, HasExecutions},
+    Error,
 };
 
 /// A per-fuzzer unique `ID`, usually starting with `0` and increasing
 /// by `1` in multiprocessed `EventManager`s, such as [`self::llmp::LlmpEventManager`].
-#[derive(Clone, Copy, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
 pub struct EventManagerId {
     /// The id
     pub id: usize,
 }
 
-impl Default for EventManagerId {
-    fn default() -> Self {
-        Self { id: 0 }
-    }
-}
-
 #[cfg(feature = "introspection")]
-use crate::stats::ClientPerfStats;
+use crate::monitors::ClientPerfMonitor;
 #[cfg(feature = "introspection")]
 use alloc::boxed::Box;
 
@@ -65,6 +69,71 @@ pub enum BrokerEventResult {
     Forward,
 }
 
+/// Distinguish a fuzzer by its config
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq)]
+pub enum EventConfig {
+    AlwaysUnique,
+    FromName {
+        name_hash: u64,
+    },
+    #[cfg(feature = "std")]
+    BuildID {
+        id: Uuid,
+    },
+}
+
+impl EventConfig {
+    #[must_use]
+    pub fn from_name(name: &str) -> Self {
+        let mut hasher = AHasher::new_with_keys(0, 0);
+        hasher.write(name.as_bytes());
+        EventConfig::FromName {
+            name_hash: hasher.finish(),
+        }
+    }
+
+    #[cfg(feature = "std")]
+    #[must_use]
+    pub fn from_build_id() -> Self {
+        EventConfig::BuildID {
+            id: build_id::get(),
+        }
+    }
+
+    #[must_use]
+    pub fn match_with(&self, other: &EventConfig) -> bool {
+        match self {
+            EventConfig::AlwaysUnique => false,
+            EventConfig::FromName { name_hash: a } => match other {
+                #[cfg(not(feature = "std"))]
+                EventConfig::AlwaysUnique => false,
+                EventConfig::FromName { name_hash: b } => (a == b),
+                #[cfg(feature = "std")]
+                EventConfig::AlwaysUnique | EventConfig::BuildID { id: _ } => false,
+            },
+            #[cfg(feature = "std")]
+            EventConfig::BuildID { id: a } => match other {
+                EventConfig::AlwaysUnique | EventConfig::FromName { name_hash: _ } => false,
+                EventConfig::BuildID { id: b } => (a == b),
+            },
+        }
+    }
+}
+
+impl From<&str> for EventConfig {
+    #[must_use]
+    fn from(name: &str) -> Self {
+        Self::from_name(name)
+    }
+}
+
+impl From<String> for EventConfig {
+    #[must_use]
+    fn from(name: String) -> Self {
+        Self::from_name(&name)
+    }
+}
+
 /*
 /// A custom event, for own messages, with own handler.
 pub trait CustomEvent<I>: SerdeAny
@@ -94,47 +163,49 @@ where
         /// The input for the new testcase
         input: I,
         /// The state of the observers when this testcase was found
-        observers_buf: Vec<u8>,
+        observers_buf: Option<Vec<u8>>,
         /// The exit kind
         exit_kind: ExitKind,
         /// The new corpus size of this client
         corpus_size: usize,
         /// The client config for this observers/testcase combination
-        client_config: String,
+        client_config: EventConfig,
         /// The time of generation of the event
         time: Duration,
         /// The executions of this client
         executions: usize,
     },
-    /// New stats.
-    UpdateStats {
+    /// New stats event to monitor.
+    UpdateExecStats {
         /// The time of generation of the [`Event`]
         time: Duration,
+        /// The stability of this fuzzer node, if known
+        stability: Option<f32>,
         /// The executions of this client
         executions: usize,
         /// [`PhantomData`]
         phantom: PhantomData<I>,
     },
-    /// New stats.
+    /// New user stats event to monitor.
     UpdateUserStats {
-        /// Custom user stats name
+        /// Custom user monitor name
         name: String,
-        /// Custom user stats value
+        /// Custom user monitor value
         value: UserStats,
         /// [`PhantomData`]
         phantom: PhantomData<I>,
     },
-    /// New stats with performance stats.
+    /// New monitor with performance monitor.
     #[cfg(feature = "introspection")]
-    UpdatePerfStats {
+    UpdatePerfMonitor {
         /// The time of generation of the event
         time: Duration,
-
         /// The executions of this client
         executions: usize,
-
+        /// The stability of this fuzzer node, if known
+        stability: Option<f32>,
         /// Current performance statistics
-        introspection_stats: Box<ClientPerfStats>,
+        introspection_monitor: Box<ClientPerfMonitor>,
 
         phantom: PhantomData<I>,
     },
@@ -174,8 +245,9 @@ where
                 time: _,
                 executions: _,
             } => "Testcase",
-            Event::UpdateStats {
+            Event::UpdateExecStats {
                 time: _,
+                stability: _,
                 executions: _,
                 phantom: _,
             }
@@ -185,12 +257,13 @@ where
                 phantom: _,
             } => "Stats",
             #[cfg(feature = "introspection")]
-            Event::UpdatePerfStats {
+            Event::UpdatePerfMonitor {
                 time: _,
                 executions: _,
-                introspection_stats: _,
+                stability: _,
+                introspection_monitor: _,
                 phantom: _,
-            } => "PerfStats",
+            } => "PerfMonitor",
             Event::Objective { objective_size: _ } => "Objective",
             Event::Log {
                 severity_level: _,
@@ -205,7 +278,7 @@ where
 }
 
 /// [`EventFirer`] fire an event.
-pub trait EventFirer<I, S>
+pub trait EventFirer<I>
 where
     I: Input,
 {
@@ -217,10 +290,28 @@ where
     /// (for example for each [`Input`], on multiple cores)
     /// the [`llmp`] shared map may fill up and the client will eventually OOM or [`panic`].
     /// This should not happen for a normal use-cases.
-    fn fire(&mut self, state: &mut S, event: Event<I>) -> Result<(), Error>;
+    fn fire<S>(&mut self, state: &mut S, event: Event<I>) -> Result<(), Error>;
+
+    /// Send off an [`Event::Log`] event to the broker
+    /// This is a shortcut for [`EventFirer::fire`] with [`Event::Log`] as argument.
+    fn log<S>(
+        &mut self,
+        state: &mut S,
+        severity_level: LogSeverity,
+        message: String,
+    ) -> Result<(), Error> {
+        self.fire(
+            state,
+            Event::Log {
+                severity_level,
+                message,
+                phantom: PhantomData,
+            },
+        )
+    }
 
     /// Serialize all observers for this type and manager
-    fn serialize_observers<OT>(&mut self, observers: &OT) -> Result<Vec<u8>, Error>
+    fn serialize_observers<OT, S>(&mut self, observers: &OT) -> Result<Vec<u8>, Error>
     where
         OT: ObserversTuple<I, S> + serde::Serialize,
     {
@@ -228,8 +319,71 @@ where
     }
 
     /// Get the configuration
-    fn configuration(&self) -> &str {
-        "<default>"
+    fn configuration(&self) -> EventConfig {
+        EventConfig::AlwaysUnique
+    }
+}
+
+/// [`EventFirer`] fire an event.
+pub trait ProgressReporter<I>: EventFirer<I>
+where
+    I: Input,
+{
+    /// Given the last time, if `monitor_timeout` seconds passed, send off an info/monitor/heartbeat message to the broker.
+    /// Returns the new `last` time (so the old one, unless `monitor_timeout` time has passed and monitor have been sent)
+    /// Will return an [`crate::Error`], if the stats could not be sent.
+    fn maybe_report_progress<S>(
+        &mut self,
+        state: &mut S,
+        last_report_time: Duration,
+        monitor_timeout: Duration,
+    ) -> Result<Duration, Error>
+    where
+        S: HasExecutions + HasClientPerfMonitor,
+    {
+        let executions = *state.executions();
+        let stability = *state.stability();
+        let cur = current_time();
+        // default to 0 here to avoid crashes on clock skew
+        if cur.checked_sub(last_report_time).unwrap_or_default() > monitor_timeout {
+            // Default no introspection implmentation
+            #[cfg(not(feature = "introspection"))]
+            self.fire(
+                state,
+                Event::UpdateExecStats {
+                    executions,
+                    stability,
+                    time: cur,
+                    phantom: PhantomData,
+                },
+            )?;
+
+            // If performance monitor are requested, fire the `UpdatePerfMonitor` event
+            #[cfg(feature = "introspection")]
+            {
+                state
+                    .introspection_monitor_mut()
+                    .set_current_time(crate::bolts::cpu::read_time_counter());
+
+                // Send the current monitor over to the manager. This `.clone` shouldn't be
+                // costly as `ClientPerfMonitor` impls `Copy` since it only contains `u64`s
+                self.fire(
+                    state,
+                    Event::UpdatePerfMonitor {
+                        executions,
+                        time: cur,
+                        stability,
+                        introspection_monitor: Box::new(state.introspection_monitor().clone()),
+                        phantom: PhantomData,
+                    },
+                )?;
+            }
+
+            Ok(cur)
+        } else {
+            if cur.as_millis() % 1000 == 0 {}
+            Ok(last_report_time)
+        }
     }
 }
 
@@ -269,7 +423,11 @@ pub trait HasEventManagerId {
 /// [`EventManager`] is the main communications hub.
 /// For the "normal" multi-processed mode, you may want to look into [`LlmpRestartingEventManager`]
 pub trait EventManager<E, I, S, Z>:
-    EventFirer<I, S> + EventProcessor<E, I, S, Z> + EventRestarter<S> + HasEventManagerId
+    EventFirer<I>
+    + EventProcessor<E, I, S, Z>
+    + EventRestarter<S>
+    + HasEventManagerId
+    + ProgressReporter<I>
 where
     I: Input,
 {
@@ -279,11 +437,11 @@ where
 #[derive(Copy, Clone, Debug)]
 pub struct NopEventManager {}
 
-impl<I, S> EventFirer<I, S> for NopEventManager
+impl<I> EventFirer<I> for NopEventManager
 where
     I: Input,
 {
-    fn fire(&mut self, _state: &mut S, _event: Event<I>) -> Result<(), Error> {
+    fn fire<S>(&mut self, _state: &mut S, _event: Event<I>) -> Result<(), Error> {
         Ok(())
     }
 }
@@ -303,6 +461,8 @@ impl<E, I, S, Z> EventProcessor<E, I, S, Z> for NopEventManager {
 
 impl<E, I, S, Z> EventManager<E, I, S, Z> for NopEventManager where I: Input {}
 
+impl<I> ProgressReporter<I> for NopEventManager where I: Input {}
+
 impl HasEventManagerId for NopEventManager {
     fn mgr_id(&self) -> EventManagerId {
         EventManagerId { id: 0 }
@@ -319,7 +479,7 @@ mod tests {
             current_time,
             tuples::{tuple_list, Named},
         },
-        events::Event,
+        events::{Event, EventConfig},
         executors::ExitKind,
         inputs::bytes::BytesInput,
         observers::StdMapObserver,
@@ -336,10 +496,10 @@ mod tests {
         let i = BytesInput::new(vec![0]);
         let e = Event::NewTestcase {
             input: i,
-            observers_buf,
+            observers_buf: Some(observers_buf),
             exit_kind: ExitKind::Ok,
             corpus_size: 123,
-            client_config: "conf".into(),
+            client_config: EventConfig::AlwaysUnique,
             time: current_time(),
             executions: 0,
         };
@@ -358,7 +518,7 @@ mod tests {
                 executions: _,
             } => {
                 let o: tuple_list_type!(StdMapObserver::<u32>) =
-                    postcard::from_bytes(&observers_buf).unwrap();
+                    postcard::from_bytes(observers_buf.as_ref().unwrap()).unwrap();
                 assert_eq!("test", o.0.name());
             }
             _ => panic!("mistmatch"),
