@@ -1,5 +1,7 @@
 //! The [`InProcessExecutor`] is a libfuzzer-like executor, that will simply call a function.
 //! It should usually be paired with extra error-handling, such as a restarting event manager, to be effective.
+//!
+//! Needs the `fork` feature flag.
 
 use core::{ffi::c_void, marker::PhantomData, ptr};
 
@@ -31,7 +33,7 @@ use crate::{
     fuzzer::HasObjective,
     inputs::Input,
     observers::ObserversTuple,
-    state::{HasClientPerfStats, HasSolutions},
+    state::{HasClientPerfMonitor, HasSolutions},
     Error,
 };
 
@@ -51,10 +53,8 @@ where
     harness_fn: &'a mut H,
     /// The observers, observing each run
     observers: OT,
-    /// On crash C function pointer
-    crash_handler: *const c_void,
-    /// On timeout C function pointer
-    timeout_handler: *const c_void,
+    // Crash and timeout hah
+    handlers: InProcessHandlers,
     phantom: PhantomData<(I, S)>,
 }
 
@@ -67,65 +67,15 @@ where
     #[inline]
     fn run_target(
         &mut self,
-        _fuzzer: &mut Z,
-        _state: &mut S,
-        _mgr: &mut EM,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
         input: &I,
     ) -> Result<ExitKind, Error> {
-        #[cfg(unix)]
-        unsafe {
-            let data = &mut GLOBAL_STATE;
-            write_volatile(
-                &mut data.current_input_ptr,
-                input as *const _ as *const c_void,
-            );
-            write_volatile(
-                &mut data.observers_ptr,
-                &self.observers as *const _ as *const c_void,
-            );
-            data.crash_handler = self.crash_handler;
-            data.timeout_handler = self.timeout_handler;
-            // Direct raw pointers access /aliasing is pretty undefined behavior.
-            // Since the state and event may have moved in memory, refresh them right before the signal may happen
-            write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
-            write_volatile(&mut data.event_mgr_ptr, _mgr as *mut _ as *mut c_void);
-            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
-            compiler_fence(Ordering::SeqCst);
-        }
-        #[cfg(all(windows, feature = "std"))]
-        unsafe {
-            let data = &mut GLOBAL_STATE;
-            write_volatile(
-                &mut data.current_input_ptr,
-                input as *const _ as *const c_void,
-            );
-            write_volatile(
-                &mut data.observers_ptr,
-                &self.observers as *const _ as *const c_void,
-            );
-            data.crash_handler = self.crash_handler;
-            data.timeout_handler = self.timeout_handler;
-            // Direct raw pointers access /aliasing is pretty undefined behavior.
-            // Since the state and event may have moved in memory, refresh them right before the signal may happen
-            write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
-            write_volatile(&mut data.event_mgr_ptr, _mgr as *mut _ as *mut c_void);
-            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
-            compiler_fence(Ordering::SeqCst);
-        }
-
+        self.handlers
+            .pre_run_target(self, fuzzer, state, mgr, input);
         let ret = (self.harness_fn)(input);
-
-        #[cfg(unix)]
-        unsafe {
-            write_volatile(&mut GLOBAL_STATE.current_input_ptr, ptr::null());
-            compiler_fence(Ordering::SeqCst);
-        }
-        #[cfg(all(windows, feature = "std"))]
-        unsafe {
-            write_volatile(&mut GLOBAL_STATE.current_input_ptr, ptr::null());
-            compiler_fence(Ordering::SeqCst);
-        }
-
+        self.handlers.post_run_target();
         Ok(ret)
     }
 }
@@ -167,72 +117,17 @@ where
         _event_mgr: &mut EM,
     ) -> Result<Self, Error>
     where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         Z: HasObjective<I, OF, S>,
     {
-        #[cfg(unix)]
-        unsafe {
-            let data = &mut GLOBAL_STATE;
-            setup_signal_handler(data)?;
-            compiler_fence(Ordering::SeqCst);
-
-            Ok(Self {
-                harness_fn,
-                observers,
-                crash_handler: unix_signal_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>
-                    as *const _,
-                timeout_handler: unix_signal_handler::inproc_timeout_handler::<
-                    EM,
-                    I,
-                    OC,
-                    OF,
-                    OT,
-                    S,
-                    Z,
-                > as *const _,
-                phantom: PhantomData,
-            })
-        }
-        #[cfg(all(windows, feature = "std"))]
-        unsafe {
-            let data = &mut GLOBAL_STATE;
-            setup_exception_handler(data)?;
-            compiler_fence(Ordering::SeqCst);
-
-            Ok(Self {
-                harness_fn,
-                observers,
-                crash_handler: windows_exception_handler::inproc_crash_handler::<
-                    EM,
-                    I,
-                    OC,
-                    OF,
-                    OT,
-                    S,
-                    Z,
-                > as *const _,
-                timeout_handler: windows_exception_handler::inproc_timeout_handler::<
-                    EM,
-                    I,
-                    OC,
-                    OF,
-                    OT,
-                    S,
-                    Z,
-                > as *const c_void,
-                // timeout_handler: ptr::null(),
-                phantom: PhantomData,
-            })
-        }
-        #[cfg(not(any(unix, all(windows, feature = "std"))))]
+        let handlers = InProcessHandlers::new::<Self, EM, I, OC, OF, OT, S, Z>()?;
         Ok(Self {
             harness_fn,
             observers,
-            crash_handler: ptr::null(),
-            timeout_handler: ptr::null(),
+            handlers,
             phantom: PhantomData,
         })
     }
@@ -250,11 +145,161 @@ where
     }
 }
 
+#[derive(Debug)]
+pub struct InProcessHandlers {
+    /// On crash C function pointer
+    pub crash_handler: *const c_void,
+    /// On timeout C function pointer
+    pub timeout_handler: *const c_void,
+}
+
+impl InProcessHandlers {
+    pub fn pre_run_target<E, EM, I, S, Z>(
+        &self,
+        executor: &E,
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) {
+        #[cfg(unix)]
+        unsafe {
+            let data = &mut GLOBAL_STATE;
+            write_volatile(
+                &mut data.current_input_ptr,
+                input as *const _ as *const c_void,
+            );
+            write_volatile(
+                &mut data.executor_ptr,
+                executor as *const _ as *const c_void,
+            );
+            data.crash_handler = self.crash_handler;
+            data.timeout_handler = self.timeout_handler;
+            // Direct raw pointers access /aliasing is pretty undefined behavior.
+            // Since the state and event may have moved in memory, refresh them right before the signal may happen
+            write_volatile(&mut data.state_ptr, state as *mut _ as *mut c_void);
+            write_volatile(&mut data.event_mgr_ptr, mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, fuzzer as *mut _ as *mut c_void);
+            compiler_fence(Ordering::SeqCst);
+        }
+        #[cfg(all(windows, feature = "std"))]
+        unsafe {
+            let data = &mut GLOBAL_STATE;
+            write_volatile(
+                &mut data.current_input_ptr,
+                input as *const _ as *const c_void,
+            );
+            write_volatile(
+                &mut data.executor_ptr,
+                executor as *const _ as *const c_void,
+            );
+            data.crash_handler = self.crash_handler;
+            data.timeout_handler = self.timeout_handler;
+            // Direct raw pointers access /aliasing is pretty undefined behavior.
+            // Since the state and event may have moved in memory, refresh them right before the signal may happen
+            write_volatile(&mut data.state_ptr, state as *mut _ as *mut c_void);
+            write_volatile(&mut data.event_mgr_ptr, mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, fuzzer as *mut _ as *mut c_void);
+            compiler_fence(Ordering::SeqCst);
+        }
+    }
+
+    #[allow(clippy::unused_self)]
+    pub fn post_run_target(&self) {
+        #[cfg(unix)]
+        unsafe {
+            write_volatile(&mut GLOBAL_STATE.current_input_ptr, ptr::null());
+            compiler_fence(Ordering::SeqCst);
+        }
+        #[cfg(all(windows, feature = "std"))]
+        unsafe {
+            write_volatile(&mut GLOBAL_STATE.current_input_ptr, ptr::null());
+            compiler_fence(Ordering::SeqCst);
+        }
+    }
+
+    pub fn new<E, EM, I, OC, OF, OT, S, Z>() -> Result<Self, Error>
+    where
+        I: Input,
+        E: HasObservers<I, OT, S>,
+        OT: ObserversTuple<I, S>,
+        EM: EventFirer<I> + EventRestarter<S>,
+        OC: Corpus<I>,
+        OF: Feedback<I, S>,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
+        Z: HasObjective<I, OF, S>,
+    {
+        #[cfg(unix)]
+        unsafe {
+            let data = &mut GLOBAL_STATE;
+            setup_signal_handler(data)?;
+            compiler_fence(Ordering::SeqCst);
+
+            Ok(Self {
+                crash_handler: unix_signal_handler::inproc_crash_handler::<E, EM, I, OC, OF, OT, S, Z>
+                    as *const _,
+                timeout_handler: unix_signal_handler::inproc_timeout_handler::<
+                    E,
+                    EM,
+                    I,
+                    OC,
+                    OF,
+                    OT,
+                    S,
+                    Z,
+                > as *const _,
+            })
+        }
+        #[cfg(all(windows, feature = "std"))]
+        unsafe {
+            let data = &mut GLOBAL_STATE;
+            setup_exception_handler(data)?;
+            compiler_fence(Ordering::SeqCst);
+
+            Ok(Self {
+                crash_handler: windows_exception_handler::inproc_crash_handler::<
+                    E,
+                    EM,
+                    I,
+                    OC,
+                    OF,
+                    OT,
+                    S,
+                    Z,
+                > as *const _,
+                timeout_handler: windows_exception_handler::inproc_timeout_handler::<
+                    E,
+                    EM,
+                    I,
+                    OC,
+                    OF,
+                    OT,
+                    S,
+                    Z,
+                > as *const c_void,
+            })
+        }
+        #[cfg(not(any(unix, all(windows, feature = "std"))))]
+        Ok(Self {
+            crash_handler: ptr::null(),
+            timeout_handler: ptr::null(),
+        })
+    }
+
+    #[must_use]
+    pub fn nop() -> Self {
+        Self {
+            crash_handler: ptr::null(),
+            timeout_handler: ptr::null(),
+        }
+    }
+}
+
 pub struct InProcessExecutorHandlerData {
     pub state_ptr: *mut c_void,
     pub event_mgr_ptr: *mut c_void,
     pub fuzzer_ptr: *mut c_void,
-    pub observers_ptr: *const c_void,
+    pub executor_ptr: *const c_void,
     pub current_input_ptr: *const c_void,
     pub crash_handler: *const c_void,
     pub timeout_handler: *const c_void,
@@ -273,8 +318,8 @@ pub static mut GLOBAL_STATE: InProcessExecutorHandlerData = InProcessExecutorHan
     event_mgr_ptr: ptr::null_mut(),
     /// The fuzzer ptr for signal handling
     fuzzer_ptr: ptr::null_mut(),
-    /// The observers ptr for signal handling
-    observers_ptr: ptr::null(),
+    /// The executor ptr for signal handling
+    executor_ptr: ptr::null(),
     /// The current input for signal handling
     current_input_ptr: ptr::null(),
     /// The crash handler fn
@@ -285,28 +330,48 @@ pub static mut GLOBAL_STATE: InProcessExecutorHandlerData = InProcessExecutorHan
     timer_queue: ptr::null_mut(),
 };
 
+#[must_use]
+pub fn inprocess_get_state<'a, S>() -> Option<&'a mut S> {
+    unsafe { (GLOBAL_STATE.state_ptr as *mut S).as_mut() }
+}
+
+#[must_use]
+pub fn inprocess_get_event_manager<'a, EM>() -> Option<&'a mut EM> {
+    unsafe { (GLOBAL_STATE.event_mgr_ptr as *mut EM).as_mut() }
+}
+
+#[must_use]
+pub fn inprocess_get_fuzzer<'a, F>() -> Option<&'a mut F> {
+    unsafe { (GLOBAL_STATE.fuzzer_ptr as *mut F).as_mut() }
+}
+
+#[must_use]
+pub fn inprocess_get_executor<'a, E>() -> Option<&'a mut E> {
+    unsafe { (GLOBAL_STATE.executor_ptr as *mut E).as_mut() }
+}
+
 #[cfg(unix)]
 mod unix_signal_handler {
     use alloc::vec::Vec;
     use core::{mem::transmute, ptr};
-    use libc::{siginfo_t, ucontext_t};
+    use libc::siginfo_t;
     #[cfg(feature = "std")]
     use std::io::{stdout, Write};
 
     use crate::{
-        bolts::os::unix_signals::{Handler, Signal},
+        bolts::os::unix_signals::{ucontext_t, Handler, Signal},
         corpus::{Corpus, Testcase},
         events::{Event, EventFirer, EventRestarter},
         executors::{
             inprocess::{InProcessExecutorHandlerData, GLOBAL_STATE},
             timeout::unix_remove_timeout,
-            ExitKind,
+            ExitKind, HasObservers,
         },
         feedbacks::Feedback,
         fuzzer::HasObjective,
         inputs::Input,
         observers::ObserversTuple,
-        state::{HasClientPerfStats, HasSolutions},
+        state::{HasClientPerfMonitor, HasMetadata, HasSolutions},
     };
 
     pub type HandlerFuncPtr =
@@ -359,122 +424,130 @@ mod unix_signal_handler {
     }
 
     #[cfg(unix)]
-    pub unsafe fn inproc_timeout_handler<EM, I, OC, OF, OT, S, Z>(
+    pub unsafe fn inproc_timeout_handler<E, EM, I, OC, OF, OT, S, Z>(
         _signal: Signal,
         _info: siginfo_t,
         _context: &mut ucontext_t,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        E: HasObservers<I, OT, S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         I: Input,
         Z: HasObjective<I, OF, S>,
     {
         let state = (data.state_ptr as *mut S).as_mut().unwrap();
         let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
         let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
-        let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
+        let executor = (data.executor_ptr as *const E).as_ref().unwrap();
+        let observers = executor.observers();
 
         if data.current_input_ptr.is_null() {
             #[cfg(feature = "std")]
-            dbg!("TIMEOUT or SIGUSR2 happened, but currently not fuzzing. Exiting");
-        } else {
-            #[cfg(feature = "std")]
-            println!("Timeout in fuzz run.");
-            #[cfg(feature = "std")]
-            let _res = stdout().flush();
-
-            let input = (data.current_input_ptr as *const I).as_ref().unwrap();
-            data.current_input_ptr = ptr::null();
-
-            let interesting = fuzzer
-                .objective_mut()
-                .is_interesting(state, event_mgr, input, observers, &ExitKind::Timeout)
-                .expect("In timeout handler objective failure.");
-
-            if interesting {
-                let mut new_testcase = Testcase::new(input.clone());
-                fuzzer
-                    .objective_mut()
-                    .append_metadata(state, &mut new_testcase)
-                    .expect("Failed adding metadata");
-                state
-                    .solutions_mut()
-                    .add(new_testcase)
-                    .expect("In timeout handler solutions failure.");
-                event_mgr
-                    .fire(
-                        state,
-                        Event::Objective {
-                            objective_size: state.solutions().count(),
-                        },
-                    )
-                    .expect("Could not send timeouting input");
-            }
-
-            event_mgr.on_restart(state).unwrap();
-
-            #[cfg(feature = "std")]
-            println!("Waiting for broker...");
-            event_mgr.await_restart_safe();
-            #[cfg(feature = "std")]
-            println!("Bye!");
-
-            event_mgr.await_restart_safe();
-
-            libc::_exit(1);
+            println!("TIMEOUT or SIGUSR2 happened, but currently not fuzzing.");
+            return;
         }
+
+        #[cfg(feature = "std")]
+        println!("Timeout in fuzz run.");
+        #[cfg(feature = "std")]
+        let _res = stdout().flush();
+
+        let input = (data.current_input_ptr as *const I).as_ref().unwrap();
+        data.current_input_ptr = ptr::null();
+
+        let interesting = fuzzer
+            .objective_mut()
+            .is_interesting(state, event_mgr, input, observers, &ExitKind::Timeout)
+            .expect("In timeout handler objective failure.");
+
+        if interesting {
+            let mut new_testcase = Testcase::new(input.clone());
+            new_testcase.add_metadata(ExitKind::Timeout);
+            fuzzer
+                .objective_mut()
+                .append_metadata(state, &mut new_testcase)
+                .expect("Failed adding metadata");
+            state
+                .solutions_mut()
+                .add(new_testcase)
+                .expect("In timeout handler solutions failure.");
+            event_mgr
+                .fire(
+                    state,
+                    Event::Objective {
+                        objective_size: state.solutions().count(),
+                    },
+                )
+                .expect("Could not send timeouting input");
+        }
+
+        event_mgr.on_restart(state).unwrap();
+
+        #[cfg(feature = "std")]
+        println!("Waiting for broker...");
+        event_mgr.await_restart_safe();
+        #[cfg(feature = "std")]
+        println!("Bye!");
+
+        event_mgr.await_restart_safe();
+
+        libc::_exit(55);
     }
 
     /// Crash-Handler for in-process fuzzing.
     /// Will be used for signal handling.
     /// It will store the current State to shmem, then exit.
     #[allow(clippy::too_many_lines)]
-    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S, Z>(
-        _signal: Signal,
+    pub unsafe fn inproc_crash_handler<E, EM, I, OC, OF, OT, S, Z>(
+        signal: Signal,
         _info: siginfo_t,
         _context: &mut ucontext_t,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        E: HasObservers<I, OT, S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         I: Input,
         Z: HasObjective<I, OF, S>,
     {
         unix_remove_timeout();
 
         #[cfg(all(target_os = "android", target_arch = "aarch64"))]
-        let _context = *(((_context as *mut _ as *mut libc::c_void as usize) + 128)
+        let _context = &mut *(((_context as *mut _ as *mut libc::c_void as usize) + 128)
             as *mut libc::c_void as *mut ucontext_t);
 
         #[cfg(feature = "std")]
-        println!("Crashed with {}", _signal);
+        eprintln!("Crashed with {}", signal);
         if data.current_input_ptr.is_null() {
             #[cfg(feature = "std")]
             {
-                println!("Double crash\n");
+                eprintln!("Double crash\n");
                 #[cfg(target_os = "android")]
                 let si_addr = (_info._pad[0] as i64) | ((_info._pad[1] as i64) << 32);
                 #[cfg(not(target_os = "android"))]
                 let si_addr = { _info.si_addr() as usize };
 
-                println!(
+                eprintln!(
                 "We crashed at addr 0x{:x}, but are not in the target... Bug in the fuzzer? Exiting.",
                 si_addr
                 );
+
+                #[cfg(all(feature = "std", unix))]
+                {
+                    let mut writer = std::io::BufWriter::new(std::io::stderr());
+                    crate::bolts::minibsod::generate_minibsod(&mut writer, signal, _info, _context)
+                        .unwrap();
+                    writer.flush().unwrap();
+                }
             }
-            // let's yolo-cat the maps for debugging, if possible.
-            #[cfg(all(target_os = "linux", feature = "std"))]
-            match std::fs::read_to_string("/proc/self/maps") {
-                Ok(maps) => println!("maps:\n{}", maps),
-                Err(e) => println!("Couldn't load mappings: {:?}", e),
-            };
+
             #[cfg(feature = "std")]
             {
                 println!("Type QUIT to restart the child");
@@ -489,47 +562,24 @@ mod unix_signal_handler {
             let state = (data.state_ptr as *mut S).as_mut().unwrap();
             let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
             let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
-            let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
-
-            #[cfg(feature = "std")]
-            println!("Child crashed!");
-
-            #[allow(clippy::non_ascii_literal)]
-            #[cfg(all(
-                feature = "std",
-                any(target_os = "linux", target_os = "android"),
-                target_arch = "aarch64"
-            ))]
-            {
-                println!("{:━^100}", " CRASH ");
-                println!(
-                    "Received signal {} at 0x{:016x}, fault address: 0x{:016x}",
-                    _signal, _context.uc_mcontext.pc, _context.uc_mcontext.fault_address
-                );
-
-                println!("{:━^100}", " REGISTERS ");
-                for reg in 0..31 {
-                    print!(
-                        "x{:02}: 0x{:016x} ",
-                        reg, _context.uc_mcontext.regs[reg as usize]
-                    );
-                    if reg % 4 == 3 {
-                        println!();
-                    }
-                }
-                println!("pc : 0x{:016x} ", _context.uc_mcontext.pc);
-
-                //println!("{:━^100}", " BACKTRACE ");
-                //println!("{:?}", backtrace::Backtrace::new())
-            }
-
-            #[cfg(feature = "std")]
-            let _res = stdout().flush();
+            let executor = (data.executor_ptr as *const E).as_ref().unwrap();
+            let observers = executor.observers();
 
             let input = (data.current_input_ptr as *const I).as_ref().unwrap();
-            // Make sure we don't crash in the crash handler forever.
             data.current_input_ptr = ptr::null();
 
+            #[cfg(feature = "std")]
+            eprintln!("Child crashed!");
+
+            #[cfg(all(feature = "std", unix))]
+            {
+                let mut writer = std::io::BufWriter::new(std::io::stderr());
+                writeln!(writer, "input: {:?}", input.generate_name(0)).unwrap();
+                crate::bolts::minibsod::generate_minibsod(&mut writer, signal, _info, _context)
+                    .unwrap();
+                writer.flush().unwrap();
+            }
+
             let interesting = fuzzer
                 .objective_mut()
                 .is_interesting(state, event_mgr, input, observers, &ExitKind::Crash)
@@ -538,6 +588,7 @@ mod unix_signal_handler {
             if interesting {
                 let new_input = input.clone();
                 let mut new_testcase = Testcase::new(new_input);
+                new_testcase.add_metadata(ExitKind::Crash);
                 fuzzer
                     .objective_mut()
                     .append_metadata(state, &mut new_testcase)
@@ -565,7 +616,7 @@ mod unix_signal_handler {
             println!("Bye!");
         }
 
-        libc::_exit(1);
+        libc::_exit(128 + (signal as i32));
     }
 }
 
@@ -589,13 +640,13 @@ mod windows_exception_handler {
         executors::{
             inprocess::{InProcessExecutorHandlerData, GLOBAL_STATE},
             timeout::windows_delete_timer_queue,
-            ExitKind,
+            ExitKind, HasObservers,
         },
         feedbacks::Feedback,
         fuzzer::HasObjective,
         inputs::Input,
         observers::ObserversTuple,
-        state::{HasClientPerfStats, HasSolutions},
+        state::{HasClientPerfMonitor, HasMetadata, HasSolutions},
     };
 
     pub type HandlerFuncPtr =
@@ -625,15 +676,16 @@ mod windows_exception_handler {
         }
     }
 
-    pub unsafe extern "system" fn inproc_timeout_handler<EM, I, OC, OF, OT, S, Z>(
+    pub unsafe extern "system" fn inproc_timeout_handler<E, EM, I, OC, OF, OT, S, Z>(
         global_state: *mut c_void,
         _p1: u8,
     ) where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        E: HasObservers<I, OT, S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         I: Input,
         Z: HasObjective<I, OF, S>,
     {
@@ -643,7 +695,8 @@ mod windows_exception_handler {
         let state = (data.state_ptr as *mut S).as_mut().unwrap();
         let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
         let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
-        let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
+        let executor = (data.executor_ptr as *const E).as_ref().unwrap();
+        let observers = executor.observers();
 
         if data.current_input_ptr.is_null() {
             #[cfg(feature = "std")]
@@ -664,6 +717,7 @@ mod windows_exception_handler {
 
             if interesting {
                 let mut new_testcase = Testcase::new(input.clone());
+                new_testcase.add_metadata(ExitKind::Timeout);
                 fuzzer
                     .objective_mut()
                     .append_metadata(state, &mut new_testcase)
@@ -697,80 +751,28 @@ mod windows_exception_handler {
         // println!("TIMER INVOKED!");
     }
 
-    pub unsafe fn inproc_crash_handler<EM, I, OC, OF, OT, S, Z>(
+    pub unsafe fn inproc_crash_handler<E, EM, I, OC, OF, OT, S, Z>(
         code: ExceptionCode,
         exception_pointers: *mut EXCEPTION_POINTERS,
         data: &mut InProcessExecutorHandlerData,
     ) where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        E: HasObservers<I, OT, S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         I: Input,
         Z: HasObjective<I, OF, S>,
     {
         // Have we set a timer_before?
-        match (data.timer_queue as *mut HANDLE).as_mut() {
-            Some(x) => {
-                windows_delete_timer_queue(*x);
-            }
-            None => {}
+        if let Some(x) = (data.timer_queue as *mut HANDLE).as_mut() {
+            windows_delete_timer_queue(*x);
         }
 
         #[cfg(feature = "std")]
         println!("Crashed with {}", code);
-        if !data.current_input_ptr.is_null() {
-            let state = (data.state_ptr as *mut S).as_mut().unwrap();
-            let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
-            let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
-            let observers = (data.observers_ptr as *const OT).as_ref().unwrap();
-
-            #[cfg(feature = "std")]
-            println!("Child crashed!");
-            #[cfg(feature = "std")]
-            let _ = stdout().flush();
-
-            let input = (data.current_input_ptr as *const I).as_ref().unwrap();
-            // Make sure we don't crash in the crash handler forever.
-            data.current_input_ptr = ptr::null();
-
-            let interesting = fuzzer
-                .objective_mut()
-                .is_interesting(state, event_mgr, &input, observers, &ExitKind::Crash)
-                .expect("In crash handler objective failure.");
-
-            if interesting {
-                let new_input = input.clone();
-                let mut new_testcase = Testcase::new(new_input);
-                fuzzer
-                    .objective_mut()
-                    .append_metadata(state, &mut new_testcase)
-                    .expect("Failed adding metadata");
-                state
-                    .solutions_mut()
-                    .add(new_testcase)
-                    .expect("In crash handler solutions failure.");
-                event_mgr
-                    .fire(
-                        state,
-                        Event::Objective {
-                            objective_size: state.solutions().count(),
-                        },
-                    )
-                    .expect("Could not send crashing input");
-            }
-
-            event_mgr.on_restart(state).unwrap();
-
-            #[cfg(feature = "std")]
-            println!("Waiting for broker...");
-            event_mgr.await_restart_safe();
-            #[cfg(feature = "std")]
-            println!("Bye!");
-
-            ExitProcess(1);
-        } else {
+        if data.current_input_ptr.is_null() {
             #[cfg(feature = "std")]
             {
                 println!("Double crash\n");
@@ -797,17 +799,67 @@ mod windows_exception_handler {
             }
 
             // TODO tell the parent to not restart
-            ExitProcess(1);
+        } else {
+            let state = (data.state_ptr as *mut S).as_mut().unwrap();
+            let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
+            let fuzzer = (data.fuzzer_ptr as *mut Z).as_mut().unwrap();
+            let executor = (data.executor_ptr as *const E).as_ref().unwrap();
+            let observers = executor.observers();
+
+            #[cfg(feature = "std")]
+            println!("Child crashed!");
+            #[cfg(feature = "std")]
+            drop(stdout().flush());
+
+            let input = (data.current_input_ptr as *const I).as_ref().unwrap();
+            // Make sure we don't crash in the crash handler forever.
+            data.current_input_ptr = ptr::null();
+
+            let interesting = fuzzer
+                .objective_mut()
+                .is_interesting(state, event_mgr, &input, observers, &ExitKind::Crash)
+                .expect("In crash handler objective failure.");
+
+            if interesting {
+                let new_input = input.clone();
+                let mut new_testcase = Testcase::new(new_input);
+                new_testcase.add_metadata(ExitKind::Crash);
+                fuzzer
+                    .objective_mut()
+                    .append_metadata(state, &mut new_testcase)
+                    .expect("Failed adding metadata");
+                state
+                    .solutions_mut()
+                    .add(new_testcase)
+                    .expect("In crash handler solutions failure.");
+                event_mgr
+                    .fire(
+                        state,
+                        Event::Objective {
+                            objective_size: state.solutions().count(),
+                        },
+                    )
+                    .expect("Could not send crashing input");
+            }
+
+            event_mgr.on_restart(state).unwrap();
+
+            #[cfg(feature = "std")]
+            println!("Waiting for broker...");
+            event_mgr.await_restart_safe();
+            #[cfg(feature = "std")]
+            println!("Bye!");
         }
+        ExitProcess(1);
     }
 }
 
 #[cfg(windows)]
-type WAITORTIMERCALLBACK = unsafe extern "system" fn(param0: *mut c_void, param1: u8);
+type WaitOrTimerCallback = unsafe extern "system" fn(param0: *mut c_void, param1: u8);
 
 #[cfg(windows)]
 pub trait HasTimeoutHandler {
-    unsafe fn timeout_handler(&self) -> WAITORTIMERCALLBACK;
+    fn timeout_handler(&self) -> WaitOrTimerCallback;
 }
 
 #[cfg(windows)]
@@ -817,9 +869,10 @@ where
     I: Input,
     OT: ObserversTuple<I, S>,
 {
+    /// the timeout handler
     #[inline]
-    unsafe fn timeout_handler(&self) -> WAITORTIMERCALLBACK {
-        let func: WAITORTIMERCALLBACK = transmute(self.timeout_handler);
+    fn timeout_handler(&self) -> WaitOrTimerCallback {
+        let func: WaitOrTimerCallback = unsafe { transmute(self.handlers.timeout_handler) };
         func
     }
 }
@@ -902,10 +955,10 @@ where
         shmem_provider: SP,
     ) -> Result<Self, Error>
     where
-        EM: EventFirer<I, S> + EventRestarter<S>,
+        EM: EventFirer<I> + EventRestarter<S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
-        S: HasSolutions<OC, I> + HasClientPerfStats,
+        S: HasSolutions<OC, I> + HasClientPerfMonitor,
         Z: HasObjective<I, OF, S>,
     {
         Ok(Self {
@@ -950,19 +1003,18 @@ where
 
 #[cfg(test)]
 mod tests {
-    use core::{marker::PhantomData, ptr};
+    use core::marker::PhantomData;
 
-    use crate::{
-        bolts::tuples::tuple_list,
-        executors::{Executor, ExitKind, InProcessExecutor},
-        inputs::NopInput,
-    };
-
-    #[cfg(all(feature = "std", unix))]
+    #[cfg(all(feature = "std", feature = "fork", unix))]
     use crate::{
         bolts::shmem::{ShMemProvider, StdShMemProvider},
         executors::InProcessForkExecutor,
     };
+    use crate::{
+        bolts::tuples::tuple_list,
+        executors::{inprocess::InProcessHandlers, Executor, ExitKind, InProcessExecutor},
+        inputs::NopInput,
+    };
 
     #[test]
     fn test_inmem_exec() {
@@ -971,8 +1023,7 @@ mod tests {
         let mut in_process_executor = InProcessExecutor::<_, NopInput, (), ()> {
             harness_fn: &mut harness,
             observers: tuple_list!(),
-            crash_handler: ptr::null(),
-            timeout_handler: ptr::null(),
+            handlers: InProcessHandlers::nop(),
             phantom: PhantomData,
         };
         let input = NopInput {};
@@ -982,7 +1033,7 @@ mod tests {
     }
 
     #[test]
-    #[cfg(all(feature = "std", unix))]
+    #[cfg(all(feature = "std", feature = "fork", unix))]
     fn test_inprocessfork_exec() {
         let provider = StdShMemProvider::new().unwrap();
 
