@@ -2,15 +2,19 @@
 
 pub mod inprocess;
 pub use inprocess::InProcessExecutor;
-#[cfg(all(feature = "std", unix))]
+#[cfg(all(feature = "std", feature = "fork", unix))]
 pub use inprocess::InProcessForkExecutor;
 
+/// Timeout executor.
+/// Not possible on `no-std` Windows or `no-std`, but works for unix
+#[cfg(any(unix, feature = "std"))]
 pub mod timeout;
+#[cfg(any(unix, feature = "std"))]
 pub use timeout::TimeoutExecutor;
 
-#[cfg(all(feature = "std", unix))]
+#[cfg(all(feature = "std", feature = "fork", unix))]
 pub mod forkserver;
-#[cfg(all(feature = "std", unix))]
+#[cfg(all(feature = "std", feature = "fork", unix))]
 pub use forkserver::{Forkserver, ForkserverExecutor, OutFile, TimeoutForkserverExecutor};
 
 pub mod combined;
@@ -22,9 +26,9 @@ pub use shadow::ShadowExecutor;
 pub mod with_observers;
 pub use with_observers::WithObservers;
 
-#[cfg(feature = "std")]
+#[cfg(all(feature = "std", unix))]
 pub mod command;
-#[cfg(feature = "std")]
+#[cfg(all(feature = "std", unix))]
 pub use command::CommandExecutor;
 
 use crate::{
@@ -36,7 +40,7 @@ use crate::{
 use serde::{Deserialize, Serialize};
 
 /// How an execution finished.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Copy, Serialize, Deserialize, PartialEq, Eq)]
 pub enum ExitKind {
     /// The run exited normally.
     Ok,
@@ -50,6 +54,8 @@ pub enum ExitKind {
     // Custom(Box<dyn SerdeAny>),
 }
 
+crate::impl_serdeany!(ExitKind);
+
 /// Holds a tuple of Observers
 pub trait HasObservers<I, OT, S>
 where
