@@ -9,6 +9,11 @@ pub mod concolic;
 #[cfg(feature = "std")]
 pub use concolic::ConcolicFeedback;
 
+#[cfg(feature = "nautilus")]
+pub mod nautilus;
+#[cfg(feature = "nautilus")]
+pub use nautilus::*;
+
 use alloc::string::{String, ToString};
 use serde::{Deserialize, Serialize};
 
@@ -19,7 +24,7 @@ use crate::{
     executors::ExitKind,
     inputs::Input,
     observers::{ObserversTuple, TimeObserver},
-    state::HasClientPerfStats,
+    state::HasClientPerfMonitor,
     Error,
 };
 
@@ -31,7 +36,7 @@ use core::{marker::PhantomData, time::Duration};
 pub trait Feedback<I, S>: Named
 where
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     /// `is_interesting ` return if an input is worth the addition to the corpus
     fn is_interesting<EM, OT>(
@@ -43,7 +48,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>;
 
     #[cfg(feature = "introspection")]
@@ -57,7 +62,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // Start a timer for this feedback
@@ -71,7 +76,7 @@ where
 
         // Add this stat to the feedback metrics
         state
-            .introspection_stats_mut()
+            .introspection_monitor_mut()
             .update_feedback(self.name(), elapsed);
 
         ret
@@ -96,18 +101,33 @@ where
 
 /// [`FeedbackState`] is the data associated with a [`Feedback`] that must persist as part
 /// of the fuzzer State
-pub trait FeedbackState: Named + serde::Serialize + serde::de::DeserializeOwned {}
+pub trait FeedbackState: Named + serde::Serialize + serde::de::DeserializeOwned {
+    /// Reset the internal state
+    fn reset(&mut self) -> Result<(), Error> {
+        Ok(())
+    }
+}
 
 /// A haskell-style tuple of feedback states
-pub trait FeedbackStatesTuple: MatchName + serde::Serialize + serde::de::DeserializeOwned {}
+pub trait FeedbackStatesTuple: MatchName + serde::Serialize + serde::de::DeserializeOwned {
+    fn reset_all(&mut self) -> Result<(), Error>;
+}
 
-impl FeedbackStatesTuple for () {}
+impl FeedbackStatesTuple for () {
+    fn reset_all(&mut self) -> Result<(), Error> {
+        Ok(())
+    }
+}
 
 impl<Head, Tail> FeedbackStatesTuple for (Head, Tail)
 where
     Head: FeedbackState,
     Tail: FeedbackStatesTuple,
 {
+    fn reset_all(&mut self) -> Result<(), Error> {
+        self.0.reset()?;
+        self.1.reset_all()
+    }
 }
 
 pub struct CombinedFeedback<A, B, I, S, FL>
@@ -116,7 +136,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     pub first: A,
     pub second: B,
@@ -130,7 +150,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name(&self) -> &str {
         self.name.as_ref()
@@ -143,7 +163,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     pub fn new(first: A, second: B) -> Self {
         let name = format!("{} ({},{})", FL::name(), first.name(), second.name());
@@ -162,7 +182,7 @@ where
     B: Feedback<I, S>,
     FL: FeedbackLogic<A, B, I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -173,7 +193,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         FL::is_pair_interesting(
@@ -197,7 +217,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         FL::is_pair_interesting_introspection(
@@ -229,7 +249,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name() -> &'static str;
 
@@ -243,7 +263,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>;
 
     #[cfg(feature = "introspection")]
@@ -258,7 +278,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>;
 }
 
@@ -272,7 +292,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name() -> &'static str {
         "Eager OR"
@@ -288,7 +308,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         let a = first.is_interesting(state, manager, input, observers, exit_kind)?;
@@ -307,7 +327,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
@@ -323,7 +343,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name() -> &'static str {
         "Fast OR"
@@ -339,7 +359,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         let a = first.is_interesting(state, manager, input, observers, exit_kind)?;
@@ -361,7 +381,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
@@ -380,7 +400,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name() -> &'static str {
         "Eager AND"
@@ -396,7 +416,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         let a = first.is_interesting(state, manager, input, observers, exit_kind)?;
@@ -415,7 +435,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
@@ -431,7 +451,7 @@ where
     A: Feedback<I, S>,
     B: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn name() -> &'static str {
         "Fast AND"
@@ -447,7 +467,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         let a = first.is_interesting(state, manager, input, observers, exit_kind)?;
@@ -469,7 +489,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // Execute this feedback
@@ -506,7 +526,7 @@ pub struct NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     /// The feedback to invert
     pub first: A,
@@ -519,7 +539,7 @@ impl<A, I, S> Feedback<I, S> for NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -530,7 +550,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         Ok(!self
@@ -553,7 +573,7 @@ impl<A, I, S> Named for NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     #[inline]
     fn name(&self) -> &str {
@@ -565,7 +585,7 @@ impl<A, I, S> NotFeedback<A, I, S>
 where
     A: Feedback<I, S>,
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     /// Creates a new [`NotFeedback`].
     pub fn new(first: A) -> Self {
@@ -578,7 +598,7 @@ where
     }
 }
 
-/// Variadic macro to create a chain of AndFeedback
+/// Variadic macro to create a chain of [`AndFeedback`](EagerAndFeedback)
 #[macro_export]
 macro_rules! feedback_and {
     ( $last:expr ) => { $last };
@@ -589,7 +609,7 @@ macro_rules! feedback_and {
     };
 }
 ///
-/// Variadic macro to create a chain of (fast) AndFeedback
+/// Variadic macro to create a chain of (fast) [`AndFeedback`](FastAndFeedback)
 #[macro_export]
 macro_rules! feedback_and_fast {
     ( $last:expr ) => { $last };
@@ -600,7 +620,7 @@ macro_rules! feedback_and_fast {
     };
 }
 
-/// Variadic macro to create a chain of OrFeedback
+/// Variadic macro to create a chain of [`OrFeedback`](EagerOrFeedback)
 #[macro_export]
 macro_rules! feedback_or {
     ( $last:expr ) => { $last };
@@ -621,7 +641,7 @@ macro_rules! feedback_or_fast {
     };
 }
 
-/// Variadic macro to create a NotFeedback
+/// Variadic macro to create a [`NotFeedback`]
 #[macro_export]
 macro_rules! feedback_not {
     ( $last:expr ) => {
@@ -633,7 +653,7 @@ macro_rules! feedback_not {
 impl<I, S> Feedback<I, S> for ()
 where
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -644,7 +664,7 @@ where
         _exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         Ok(false)
@@ -665,7 +685,7 @@ pub struct CrashFeedback {}
 impl<I, S> Feedback<I, S> for CrashFeedback
 where
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -676,7 +696,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         if let ExitKind::Crash = exit_kind {
@@ -715,7 +735,7 @@ pub struct TimeoutFeedback {}
 impl<I, S> Feedback<I, S> for TimeoutFeedback
 where
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -726,7 +746,7 @@ where
         exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         if let ExitKind::Timeout = exit_kind {
@@ -770,7 +790,7 @@ pub struct TimeFeedback {
 impl<I, S> Feedback<I, S> for TimeFeedback
 where
     I: Input,
-    S: HasClientPerfStats,
+    S: HasClientPerfMonitor,
 {
     fn is_interesting<EM, OT>(
         &mut self,
@@ -781,7 +801,7 @@ where
         _exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        EM: EventFirer<I, S>,
+        EM: EventFirer<I>,
         OT: ObserversTuple<I, S>,
     {
         // TODO Replace with match_name_type when stable
