@@ -48,6 +48,7 @@ impl CmpValues {
 #[derive(Default, Serialize, Deserialize)]
 pub struct CmpValuesMetadata {
     /// A `list` of values.
+    #[serde(skip)]
     pub list: Vec<CmpValues>,
 }
 
@@ -107,7 +108,7 @@ where
     /// Get the `CmpMap` (mut)
     fn map_mut(&mut self) -> &mut CM;
 
-    /// Add [`CmpValuesMetadata`] to the State including the logged values.
+    /// Add [`struct@CmpValuesMetadata`] to the State including the logged values.
     /// This routine does a basic loop filtering because loop index cmps are not interesting.
     fn add_cmpvalues_meta(&mut self, state: &mut S)
     where
