@@ -43,10 +43,7 @@
 
 #![cfg(feature = "std")]
 
-use std::{
-    convert::TryFrom,
-    io::{self, Cursor, Read, Seek, SeekFrom, Write},
-};
+use std::io::{self, Cursor, Read, Seek, SeekFrom, Write};
 
 use bincode::{DefaultOptions, Options};
 
@@ -198,6 +195,7 @@ impl<R: Read> MessageFileReader<R> {
                     *expr = self.make_absolute(*expr);
                 }
             }
+            SymExpr::Call { .. } | SymExpr::Return { .. } | SymExpr::BasicBlock { .. } => {}
         }
         SymExprRef::new(ret).unwrap()
     }
@@ -348,6 +346,7 @@ impl<W: Write + Seek> MessageFileWriter<W> {
                     *expr = self.make_relative(*expr);
                 }
             }
+            SymExpr::Call { .. } | SymExpr::Return { .. } | SymExpr::BasicBlock { .. } => {}
         }
         self.serialization_options
             .serialize_into(&mut self.writer, &message)?;
