@@ -19,7 +19,7 @@ use crate::{
     fuzzer::{Evaluator, ExecuteInputResult},
     generators::Generator,
     inputs::Input,
-    stats::ClientPerfStats,
+    monitors::ClientPerfMonitor,
     Error,
 };
 
@@ -71,13 +71,19 @@ where
     fn rand_mut(&mut self) -> &mut R;
 }
 
-/// Trait for offering a [`ClientPerfStats`]
-pub trait HasClientPerfStats {
-    /// [`ClientPerfStats`] itself
-    fn introspection_stats(&self) -> &ClientPerfStats;
+/// Trait for offering a [`ClientPerfMonitor`]
+pub trait HasClientPerfMonitor {
+    /// [`ClientPerfMonitor`] itself
+    fn introspection_monitor(&self) -> &ClientPerfMonitor;
 
-    /// Mutatable ref to [`ClientPerfStats`]
-    fn introspection_stats_mut(&mut self) -> &mut ClientPerfStats;
+    /// Mutatable ref to [`ClientPerfMonitor`]
+    fn introspection_monitor_mut(&mut self) -> &mut ClientPerfMonitor;
+
+    /// This node's stability
+    fn stability(&self) -> &Option<f32>;
+
+    /// This node's stability (mut)
+    fn stability_mut(&mut self) -> &mut Option<f32>;
 }
 
 /// Trait for elements offering metadata
@@ -163,10 +169,12 @@ where
     metadata: SerdeAnyMap,
     /// MaxSize testcase size for mutators that appreciate it
     max_size: usize,
+    /// The stability of the current fuzzing process
+    stability: Option<f32>,
 
     /// Performance statistics for this fuzzer
     #[cfg(feature = "introspection")]
-    introspection_stats: ClientPerfStats,
+    introspection_monitor: ClientPerfMonitor,
 
     phantom: PhantomData<I>,
 }
@@ -357,13 +365,14 @@ where
     /// loads inputs from a directory
     /// If `forced` is `true`, the value will be loaded,
     /// even if it's not considered to be `interesting`.
-    fn load_from_directory<E, EM, Z>(
+    pub fn load_from_directory<E, EM, Z>(
         &mut self,
         fuzzer: &mut Z,
         executor: &mut E,
         manager: &mut EM,
         in_dir: &Path,
         forced: bool,
+        loader: &mut dyn FnMut(&mut Z, &mut Self, &Path) -> Result<I, Error>,
     ) -> Result<(), Error>
     where
         Z: Evaluator<E, EM, I, Self>,
@@ -381,7 +390,7 @@ where
 
             if attr.is_file() && attr.len() > 0 {
                 println!("Loading file {:?} ...", &path);
-                let input = I::from_file(&path)?;
+                let input = loader(fuzzer, self, &path)?;
                 if forced {
                     let _ = fuzzer.add_input(self, executor, manager, input)?;
                 } else {
@@ -391,7 +400,7 @@ where
                     }
                 }
             } else if attr.is_dir() {
-                self.load_from_directory(fuzzer, executor, manager, &path, forced)?;
+                self.load_from_directory(fuzzer, executor, manager, &path, forced, loader)?;
             }
         }
 
@@ -410,10 +419,17 @@ where
     ) -> Result<(), Error>
     where
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         for in_dir in in_dirs {
-            self.load_from_directory(fuzzer, executor, manager, in_dir, forced)?;
+            self.load_from_directory(
+                fuzzer,
+                executor,
+                manager,
+                in_dir,
+                forced,
+                &mut |_, _, path| I::from_file(&path),
+            )?;
         }
         manager.fire(
             self,
@@ -438,7 +454,7 @@ where
     ) -> Result<(), Error>
     where
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         self.load_initial_inputs_internal(fuzzer, executor, manager, in_dirs, true)
     }
@@ -453,7 +469,7 @@ where
     ) -> Result<(), Error>
     where
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         self.load_initial_inputs_internal(fuzzer, executor, manager, in_dirs, false)
     }
@@ -477,13 +493,13 @@ where
         forced: bool,
     ) -> Result<(), Error>
     where
-        G: Generator<I, R>,
+        G: Generator<I, Self>,
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         let mut added = 0;
         for _ in 0..num {
-            let input = generator.generate(self.rand_mut())?;
+            let input = generator.generate(self)?;
             if forced {
                 let _ = fuzzer.add_input(self, executor, manager, input)?;
                 added += 1;
@@ -515,9 +531,9 @@ where
         num: usize,
     ) -> Result<(), Error>
     where
-        G: Generator<I, R>,
+        G: Generator<I, Self>,
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         self.generate_initial_internal(fuzzer, executor, generator, manager, num, true)
     }
@@ -532,9 +548,9 @@ where
         num: usize,
     ) -> Result<(), Error>
     where
-        G: Generator<I, R>,
+        G: Generator<I, Self>,
         Z: Evaluator<E, EM, I, Self>,
-        EM: EventFirer<I, Self>,
+        EM: EventFirer<I>,
     {
         self.generate_initial_internal(fuzzer, executor, generator, manager, num, false)
     }
@@ -544,6 +560,7 @@ where
         Self {
             rand,
             executions: 0,
+            stability: None,
             start_time: Duration::from_millis(0),
             metadata: SerdeAnyMap::default(),
             corpus,
@@ -551,14 +568,14 @@ where
             solutions,
             max_size: DEFAULT_MAX_SIZE,
             #[cfg(feature = "introspection")]
-            introspection_stats: ClientPerfStats::new(),
+            introspection_monitor: ClientPerfMonitor::new(),
             phantom: PhantomData,
         }
     }
 }
 
 #[cfg(feature = "introspection")]
-impl<C, FT, I, R, SC> HasClientPerfStats for StdState<C, FT, I, R, SC>
+impl<C, FT, I, R, SC> HasClientPerfMonitor for StdState<C, FT, I, R, SC>
 where
     C: Corpus<I>,
     I: Input,
@@ -566,17 +583,29 @@ where
     FT: FeedbackStatesTuple,
     SC: Corpus<I>,
 {
-    fn introspection_stats(&self) -> &ClientPerfStats {
-        &self.introspection_stats
+    fn introspection_monitor(&self) -> &ClientPerfMonitor {
+        &self.introspection_monitor
     }
 
-    fn introspection_stats_mut(&mut self) -> &mut ClientPerfStats {
-        &mut self.introspection_stats
+    fn introspection_monitor_mut(&mut self) -> &mut ClientPerfMonitor {
+        &mut self.introspection_monitor
+    }
+
+    /// This node's stability
+    #[inline]
+    fn stability(&self) -> &Option<f32> {
+        &self.stability
+    }
+
+    /// This node's stability (mut)
+    #[inline]
+    fn stability_mut(&mut self) -> &mut Option<f32> {
+        &mut self.stability
     }
 }
 
 #[cfg(not(feature = "introspection"))]
-impl<C, FT, I, R, SC> HasClientPerfStats for StdState<C, FT, I, R, SC>
+impl<C, FT, I, R, SC> HasClientPerfMonitor for StdState<C, FT, I, R, SC>
 where
     C: Corpus<I>,
     I: Input,
@@ -584,11 +613,23 @@ where
     FT: FeedbackStatesTuple,
     SC: Corpus<I>,
 {
-    fn introspection_stats(&self) -> &ClientPerfStats {
+    fn introspection_monitor(&self) -> &ClientPerfMonitor {
         unimplemented!()
     }
 
-    fn introspection_stats_mut(&mut self) -> &mut ClientPerfStats {
+    fn introspection_monitor_mut(&mut self) -> &mut ClientPerfMonitor {
         unimplemented!()
     }
+
+    /// This node's stability
+    #[inline]
+    fn stability(&self) -> &Option<f32> {
+        &self.stability
+    }
+
+    /// This node's stability (mut)
+    #[inline]
+    fn stability_mut(&mut self) -> &mut Option<f32> {
+        &mut self.stability
+    }
 }
